#include "server_util.h"
#include <assert.h>
#include <errno.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>
#include <iostream>

using namespace std;
#define BUFFER_SIZE 4096

int WrappedRead(int fd, string *file_path) {
  int read_res;
  int bytes_read = 0;
  char buffer[BUFFER_SIZE];
  while (1) {
    read_res = read(fd, buffer, BUFFER_SIZE);
    // check for reading errors
    if (read_res == -1) {
      // check if error is recoverable, if so, try again
      if ((errno == EAGAIN) || (errno == EINTR))
        continue;


    }
    // we read readlen bytes, so stop reading
    bytes_read += read_res;

    if (read_res == 0) {
      break;
    }

    bool found_endline = false;
    file_path->append((char*)buffer, read_res);
    int endline_idx = file_path->find('\n');

    if (endline_idx != -1) {
      *file_path = file_path->substr(0, endline_idx);
      break;
    }
  }
  return bytes_read;
}


int WrappedWrite(int fd, unsigned char *buf, int writelen) {
  int total = 0;
  int wlen;
  while (total < writelen) {
    wlen = write(fd, buf + total, writelen - total);
    if (wlen == -1) {
      // check if recoverable, if it is, try again
      if ((errno == EAGAIN) || (errno == EINTR))
        continue;
      // unrecoverable, stop writing.
      break;
    }
    // nothing left to write, stop writing
    if (wlen == 0)
      break;
    total += wlen;
  }
  return total;
}

bool IsPathSafe(string rootdir, string testfile) {
  char rootpathbuffer[PATH_MAX], testfilebuffer[PATH_MAX];
  if (!realpath(rootdir.c_str(), rootpathbuffer))
    return false;
  if (!realpath(testfile.c_str(), testfilebuffer))
    return false;

  // Make sure that testfile is a prefix of rootdir.
  if (strlen(testfilebuffer) <= strlen(rootpathbuffer))
    return false;
  if (strncmp(rootpathbuffer,
              testfilebuffer,
              strlen(rootpathbuffer)) != 0)
    return false;

  // It's safe!
  return true;
}
