class Message:
    def __init__(self, addr) -> None:
        self.addr = addr # address of sender

    def __str__(self) -> str:
        return "addr: " + str(self.addr)

class P1A(Message):
    def __init__(self, addr, ballot_num) -> None:
        super().__init__(addr)
        # ballot_num is a BallotNumber
        self.ballot_num = ballot_num

    def __str__(self) -> str:
        return "P1A(" + super().__str__() + ", " + str(self.ballot_num) + ")\n"

class P1B(Message):
    def __init__(self, addr, accepted_ballot, accepted) -> None:
        super().__init__(addr)
        # accepted_ballot is a BallotNumber
        # accepted is a dict from int to BallotValuePair
        self.accepted_ballot = accepted_ballot
        self.accepted = accepted

    def __str__(self) -> str:
        return "P1B(" + super().__str__() + ", " + str(self.accepted_ballot) + ", " + str(self.accepted) + ")\n"


class P2A(Message):
    def __init__(self, addr, ballot, slot_num, value) -> None:
        super().__init__(addr)
        # ballot is BallotNumber
        # slot_num is an int
        # value is a PaxosRequest
        self.ballot = ballot
        self.slot_num = slot_num
        self.value = value

    def __str__(self) -> str:
        return "P2A(" + super().__str__() + ", " + str(self.ballot) + ", slot_num: " + str(self.slot_num) + ", value: " + str(self.value) + ")\n"

class P2B(Message):
    def __init__(self, addr, ballot, slot_num) -> None:
        super().__init__(addr)
        # ballot is BallotNumber
        # slot_num is an int
        self.ballot = ballot
        self.slot_num = slot_num

    def __str__(self) -> str:
        return "P2B(" + super().__str__() + ", " + str(self.ballot) + ", slot_num: " + str(self.slot_num) +  ")\n"

# TODO: need to add heartbeats

## Client messages=, separate into a new class

class PaxosRequest(Message):
    def __init__(self, addr, lock_command) -> None:
        super().__init__(addr)
        # lock_command is of type LockCommand
        self.lock_command = lock_command

    def __str__(self) -> str:
        return "PaxosRequest(" + super().__str__() + ", " + str(self.lock_command) + ")\n"

class PaxosResult(Message):
    def __init__(self, addr, value) -> None:
        super().__init__(addr)
        self.value = value

    def __str__(self) -> str:
        return "PaxosResult(" + super().__str__() + ", value: " + str(self.value) + ")\n"

class LockCommand:
    def __init__(self, op, value) -> None:
        # op is either "lock" or "unlock"
        # value is value to lock (int)
        self.op = op
        self.value = value
    
    def __str__(self) -> str:
        return "LockCommand(op: " + str(self.op) + ", value: " + str(self.value) + ")"